Package: graphlayouts
Title: Additional Layout Algorithms for Network Visualizations
Version: 1.2.4
Authors@R: person("David", "Schoch", email = "david@schochastics.net", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2952-4812"))
Description: Several new layout algorithms to visualize networks are provided which are not part of 'igraph'. 
    Most are based on the concept of stress majorization by Gansner et al. (2004) <doi:10.1007/978-3-540-31843-9_25>. 
    Some more specific algorithms allow the user to emphasize hidden group structures in networks or focus on specific nodes.
URL: https://github.com/schochastics/graphlayouts,
        https://schochastics.github.io/graphlayouts/
BugReports: https://github.com/schochastics/graphlayouts/issues
Depends: R (>= 3.6.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: igraph (>= 2.0.0), Rcpp
Suggests: testthat, ggplot2, uwot, oaqc
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-06-18 18:23:41 UTC; david
Author: David Schoch [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2952-4812>)
Maintainer: David Schoch <david@schochastics.net>
Repository: CRAN
Date/Publication: 2026-06-19 05:10:17 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-19 07:08:09 UTC; unix
Archs: graphlayouts.so.dSYM
