#' Multilevel example Network
#' @description Multilevel network, where both levels have different structural features

#' @format igraph object
"multilvl_ex"

#' Subway network of Berlin
#'
#' A dataset containing the subway network of Berlin
#' @format igraph object
#' @references
#' Kujala, Rainer, et al. "A collection of public transport network data sets for 25 cities." Scientific data 5 (2018): 180089.
"metro_berlin"
